@@ -14,8 +14,13 @@ class BaseStorageConnector(ABC):
         """Initialise the storage backend whatever it is."""
 
     @abstractmethod
-    def write_resource(self, urn: AwsUrn, resource: str) -> None:
-        """Persist a single resource to storage."""
+    def write_resource(self, urn: AwsUrn, resource: boto3.resources.model.ResourceModel) -> None:
+        """Persist a single resource to storage.
+
+        Arguments:
+            urn (AwsUrn): The URN of the resource to write.
+            resource (boto3.resources.model.ResourceModel): The boto3 resource to write.
+        """
 
     @abstractmethod
     def read_all(self) -> Iterator[dict]:
@@ -23,17 +28,36 @@ class BaseStorageConnector(ABC):
 
     @abstractmethod
     def read_resource(self, urn: AwsUrn) -> 'CloudWandererResource':
-        """Return a resource matching the supplied urn from storage."""
+        """Return a resource matching the supplied urn from storage.
+
+        Arguments:
+            urn (AwsUrn): The AWS URN of the resource to return
+        """
 
     @abstractmethod
-    def read_resources(
-            self, urn: AwsUrn, account_id: str, region: str, service: str,
-            resource_type: str) -> Iterator['CloudWandererResource']:
-        """Yield a resource matching the supplied urn from storage."""
+    def read_resources(self, **kwargs) -> Iterator['CloudWandererResource']:
+        """Yield a resource matching the supplied urn from storage.
+
+        All arguments are optional.
+
+        Arguments:
+            kwargs: Optional arguments narrowing the scope of the resources returned.
+
+        Keyword Arguments:
+            urn (~cloudwanderer.aws_urn.AwsUrn): The AWS URN of the resource to return
+            account_id (:class:`str`): AWS Account ID
+            region (:class:`str`): AWS region (e.g. ``'eu-west-2'``)
+            service (:class:`str`): Service name (e.g. ``'ec2'``)
+            resource_type (:class:`str`): Resource Type (e.g. ``'instance'``)
+        """
 
     @abstractmethod
     def delete_resource(self, urn: AwsUrn) -> None:
-        """Delete this resource and all its resource attributes."""
+        """Delete this resource and all its resource attributes.
+
+        Arguments:
+            urn (AwsUrn): The URN of the resource to delete
+        """
 
     @abstractmethod
     def delete_resource_of_type_in_account_region(
@@ -42,11 +66,18 @@ class BaseStorageConnector(ABC):
         """Delete resources of type in account and region unless in list of URNs.
 
         This is used primarily to clean up old resources.
+
+        Arguments:
+            account_id (str): AWS Account ID
+            region (str): AWS region (e.g. ``'eu-west-2'``)
+            service (str): Service name (e.g. ``'ec2'``)
+            resource_type (str): Resource Type (e.g. ``'instance'``)
+            urns_to_keep (List[cloudwanderer.aws_urn.AwsUrn]): A list of resources not to delete
         """
     @abstractmethod
     def write_secondary_attribute(
             self, urn: AwsUrn, attribute_type: str, secondary_attribute: boto3.resources.base.ServiceResource) -> None:
-        """Write the specified resource attribute to DynamoDb.
+        """Write the specified resource attribute to storage.
 
         Arguments:
             urn (AwsUrn): The resource whose attribute to write.
